@@ -18,6 +18,9 @@ def manual_authenticator(user: User) -> AuthResponse:
         'tech': AuthTech.MANUAL,
     })
 
+    print('#' * 80)
+    print(user)
+
     headers = user.headers
 
     if not headers:
